{ TMixerVolume version 1.1 copyright  by Alexander Meeder, 1996

  This component is an interface for the Windows API:
  * auxGetNumDevs
  * auxGetDevCaps
  * auxGetVolume
  * auxSetVolume

  I first subclassed TScrollBar but later I made it an invisible component so you can use it
  with 'ScrollBar', 'TrackBar' etc. as you like

  There might be one problem, I don't know if the type TDeviceTypes is correct for every machine/
  situation, and if not, what do I have to do to determine which DeviceID stands for which device?

  properties
  * Channel (both, left or right)
  * DeviceType (CDAudio, Midi, Wave etc.)
  * DriverVersion (versionnumber of the installed Mixerdriver)
  * Manufacturer (Microsoft or # = Unknown)
  * Mute (volume 0, can't change volume)
  * NumOfDevices (readonly property to readout available number of devices)
  * ProductName (Productname/Device info.)
  * SupportChannels (can choose left/right if true)
  * SupportChannels (can change volume if true)
  * Volume (0 .. 255 = min .. max volume)

  method
  * Update (update property value's, especialy the volume) 

  You can use it for free, if you make changes or have suggestions,
  please mail me: ameeder@dds.nl
}

unit volume;

interface

uses
  Windows, {Messages,} SysUtils, Classes, MMSystem;

type
  EDeviceTypeError = class(Exception);
  TDeviceTypes = (dtWave, dtMidi, dtCDAudio, dtLineIn, dtMicrophone, dTMixerVolume, dtPCSpeaker);
  TChannel = (chLeft, chRight, chBoth);
  TMinMaxVolume = 0 .. 255;

  TMixerVolume = class(TComponent)
  private
    FChannel: TChannel;
    FDeviceType: TDeviceTypes;
    FDriverVersion: MMVERSION;
    FManufacturer: string;
    FMute: boolean;
    FNumberOfDevices: integer;
    FProductName: string;
    FSupportChannels: boolean;
    FSupportVolume: boolean;
    FVolume: TMinMaxVolume;

    function VolumeCount(pos: byte): longint;
    function ReadVolume: TMinMaxVolume;
    function GetNumberOfDevices: integer;
    procedure ReadDevCaps(Dev: TDeviceTypes);
    procedure SetChannel(value: TChannel);
    procedure SetDeviceType(value: TDeviceTypes);
    procedure SetMute(value: boolean);
    procedure SetVolume(value: TMinMaxVolume);
  protected
    DummyDevices: integer;
    DummySupport: boolean;
    DummyManufacturer: string;
    DummyDriverVersion: MMVERSION;
    DummyProductName: string;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Update;
  published
    property DeviceType: TDeviceTypes read FDeviceType write SetDeviceType;
    property Channel: TChannel read FChannel write SetChannel default chBoth;
    property DriverVersion: MMVERSION read FDriverVersion write DummyDriverVersion;
    property Manufacturer: string read FManufacturer write DummyManufacturer;
    property Mute: boolean read FMute write SetMute default false;
    property NumberOfDevices: integer read GetNumberOfDevices write DummyDevices;
    property ProductName: string read FProductName write DummyProductName;
    property SupportChannels: boolean read FSupportChannels write DummySupport;
    property SupportVolume: boolean read FSupportVolume write DummySupport;
    property Volume: TMinMaxVolume read FVolume write SetVolume default 25;
  end;

procedure Register;

implementation

function TMixerVolume.VolumeCount(pos: byte): longint;
begin
  case Channel of
     chBoth  : result := ((pos shl 8) or (pos shl 24));
     chLeft  : result := (pos or (pos shl 8));
     chRight : result := ((pos shl 16) or (pos shl 24));
  end;
end;

function TMixerVolume.ReadVolume: TMinMaxVolume;
var
  ReadVol: integer;
  DeviceErrMsg: string;
begin
    if auxGetVolume(ord(DeviceType), @ReadVol) = MMSYSERR_BADDEVICEID then
    begin
      case DeviceType of
        dtWave        : DeviceErrMsg := 'No Wave device available';
        dtMidi        : DeviceErrMsg := 'No Midi device available';
        dtCDAudio     : DeviceErrMsg := 'No CD-audio device available';
        dtLineIn      : DeviceErrMsg := 'No Line-In device available';
        dtMicrophone  : DeviceErrMsg := 'No Microfoon device avialable';
        dTMixerVolume : DeviceErrMsg := 'No Volume device available';
        dtPCSpeaker   : DeviceErrMsg := 'No Speaker device available';
      end;
      raise EDeviceTypeError.Create(DeviceErrMsg);
    end
    else
    if (Channel = chLeft) or (Channel = chBoth) then
       Result := Hi(ReadVol)
    else
       Result := Hi(ReadVol shr 8);
end;

procedure TMixerVolume.ReadDevCaps(Dev: TDeviceTypes);
var
  auxCaps: TAuxCaps;
  DeviceErrMsg: string;
begin
  if auxGetDevCaps(ord(Dev), @auxCaps, SizeOf(auxCaps)) = MMSYSERR_BADDEVICEID then
  begin
      case DeviceType of
        dtWave        : DeviceErrMsg := 'No Wave device available';
        dtMidi        : DeviceErrMsg := 'No Midi device available';
        dtCDAudio     : DeviceErrMsg := 'No CD-audio device available';
        dtLineIn      : DeviceErrMsg := 'No Line-In device available';
        dtMicrophone  : DeviceErrMsg := 'No Microfoon device avialable';
        dTMixerVolume : DeviceErrMsg := 'No Volume device available';
        dtPCSpeaker   : DeviceErrMsg := 'No Speaker device available';
      end;
      raise EDeviceTypeError.Create(DeviceErrMsg);
  end
  else
  begin
    if (auxCaps.dwSupport and AUXCAPS_VOLUME) = 1 then
       FSupportVolume := true else FSupportVolume := false;
    if (auxCaps.dwSupport and AUXCAPS_LRVOLUME) = 2 then
       FSupportChannels := true else FSupportChannels := false;
    if auxCaps.wMid = MM_MICROSOFT then
       FManufacturer := 'Microsoft' else FManufacturer := IntToStr(auxCaps.wMid) + ' = Unknown';
    FDriverVersion := auxCaps.vDriverVersion;
    FProductName := StrPas(auxCaps.szPName);
  end;
end;

procedure TMixerVolume.Update;
begin
  ReadDevCaps(DeviceType);
  FVolume := ReadVolume;
end;

procedure TMixerVolume.SetChannel(value: TChannel);
begin
  if (value <> FChannel) and (FSupportChannels) then
  begin
    FChannel := value;
    ReadVolume;
  end
  else
    FChannel := chBoth;
end;

procedure TMixerVolume.SetDeviceType(value: TDeviceTypes);
begin
  ReadDevCaps(value);
  if (value <> FDeviceType) and (FSupportVolume) then
  begin
    FDeviceType := value;
    FVolume := ReadVolume;
    if not FSupportChannels then FChannel := chBoth;
  end;
end;

procedure TMixerVolume.SetMute(value: boolean);
begin
  if value <> FMute then
  begin
    FMute := value;
    if FMute then Volume := 0;
  end;
end;

procedure TMixerVolume.SetVolume(value: TMinMaxVolume);
var
  DeviceErrMsg: string;
begin
  if (value <> FVolume) and not (Mute) then
  begin
    FVolume := value;
    if auxSetVolume(ord(DeviceType), VolumeCount(FVolume)) = MMSYSERR_BADDEVICEID then
    begin
      case DeviceType of
        dtWave        : DeviceErrMsg := 'No Wave device available';
        dtMidi        : DeviceErrMsg := 'No Midi device available';
        dtCDAudio     : DeviceErrMsg := 'No CD-audio device available';
        dtLineIn      : DeviceErrMsg := 'No Line-In device available';
        dtMicrophone  : DeviceErrMsg := 'No Microfoon device avialable';
        dTMixerVolume : DeviceErrMsg := 'No Volume device available';
        dtPCSpeaker   : DeviceErrMsg := 'No Speaker device available';
      end;
      raise EDeviceTypeError.Create(DeviceErrMsg);
    end;
  end;
end;

function TMixerVolume.GetNumberOfDevices: integer;
begin
  result := auxGetNumDevs;
end;

procedure TMixerVolume.Loaded;

begin
  inherited Loaded;
  FVolume := ReadVolume;
  ReadDevCaps(FDeviceType);
end;

constructor TMixerVolume.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FChannel := chBoth;
  // FDeviceType := dtCDAudio;
  FMute := false;
  FVolume := ReadVolume;
  ReadDevCaps(DeviceType);
end;

destructor TMixerVolume.Destroy;
begin
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TMixerVolume]);
end;

end.
